'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _position = require('./position/');

var _position2 = _interopRequireDefault(_position);

var _isAuto = require('./position/isAuto');

var _isAuto2 = _interopRequireDefault(_isAuto);

var _stretch = require('./position/stretch');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (rule) {

    var position = _position2.default.parse(rule).stretch();

    rule.append({
        prop: 'width',
        value: position.horizontal.size
    }, {
        prop: 'height',
        value: position.vertical.size
    });

    if (position.align) {

        // display: block or inline-block depending on the axis
        rule.append({
            prop: 'display',
            value: position.align.display
        });

        // margins
        position.iterateDirections(function (direction, lengths) {
            rule.append({
                prop: 'margin-' + direction.before,
                value: lengths[0]
            }, {
                prop: 'margin-' + direction.after,
                value: lengths[2]
            });
        });

        // first and last margins
        if (position.margins) {

            rule.after({
                selector: rule.selector + ':first-child'
            }).next().append({
                prop: 'margin-' + position.align.before,
                value: position.margins.first
            }).after({
                selector: rule.selector + ':last-child'
            }).next().append({
                prop: 'margin-' + position.align.after,
                value: position.margins.last
            });
        }
    } else {

        var relative = false;
        position.iterateDirections(function (direction, lengths) {
            if (lengths.some(_isAuto2.default) && (0, _stretch.stretchCount)(lengths) === 2) {
                position.type = 'relative';

                var ratio = (0, _stretch.stretchRatio)(lengths);
                rule.append({
                    prop: direction.before,
                    value: (0, _stretch.wrapCalc)(ratio)
                });

                var transformValue = direction.transform + '(' + (0, _stretch.wrapCalc)('-' + ratio) + ')';
                if (relative) {
                    var transform = rule.nodes.filter(function (node) {
                        return node.prop === 'transform';
                    })[0];
                    transform.value += ' ' + transformValue;
                } else {
                    rule.append({
                        prop: 'transform',
                        value: transformValue
                    });
                    relative = true;
                }
            } else {
                // absolute space
                rule.append({
                    prop: direction.before,
                    value: lengths[0]
                });
            }
        });

        if (relative) {
            rule.append({
                prop: 'display',
                value: 'inline-block'
            });
        }

        // position: absolute / relative / static / fixed
        if (position.type) {
            rule.append({
                prop: 'position',
                value: position.type
            });
        } else {
            rule.append({
                prop: 'position',
                value: 'absolute'
            });
        }
    }
}; // Processes positioning for a rule